function [success,normgrad,objective,iter,inner_iter,time] = btd_gradVSbtd_grad_full(I,R,Rank,sigma,tol,tol_obj,itermax_grad,itermax_grad_full,sample_A,sample_initial)
%% Description
% Riemannian gradient algorithm for BTD with and without variable projection.
% This function builds noised BTDs and then applies the Riemannian gradient
% algorithm with (btd_grad) and without (btd_grad_full) variable projection
% starting from the same starting iterates.
% --- BTD parameters ---
% I = size of the tensor
% R = number of terms in the BTD
% Rank = multilinear rank of each term in the BTD
% sigma = norm of the noise added on the BTD (the BTD has unit norm)
% --- Precision parameters ---
% tol = tolerance on the norm of the gradient; tol has to be chosen according to the 4 previous parameters, e.g., sigma = 0.1 -> tol = 4e-8; sigma = 0.5 -> tol = 2e-7; sigma = 1 -> tol = 3e-7;
% tol_obj = tolerance on the objective function; will be used if and only if sigma = 0
% itermax_grad = maximum number of iterations performed by btd_grad
% itermax_grad_full = maximum number of iterations performed by btd_grad_full
% --- Sample size ---
% sample_A = number of A
% sample_initial = number of initial iterate(s) for each A
%% Raw data
sample = sample_A*sample_initial;
% Column 1 = grad, Column 2 = grad_full
success = [0 0]; % success means that both normgrad < tol and objective < tol_obj
normgrad = zeros(sample,2); % norm of the gradient at the end of the process
objective = zeros(sample,2); % objective at the end of the process
iter = zeros(sample,2); % number of iterations needed to converge
inner_iter = zeros(sample,2); % mean number of iterations in the backtracking loop
time = zeros(sample,3); % Column 3 = time spent in variable projection
%% Main
for s_A = 1 : sample_A
    % Building the BTD
    A0 = zeros(I);
    S = randn([Rank R]);
    U = zeros(I(1),Rank(1),R);
    V = zeros(I(2),Rank(2),R);
    W = zeros(I(3),Rank(3),R);
    for r = 1:R
        U(:,:,r) = qf(randn(I(1),Rank(1)));
        V(:,:,r) = qf(randn(I(2),Rank(2)));
        W(:,:,r) = qf(randn(I(3),Rank(3)));
        A0 = A0 + tmprod(S(:,:,:,r),{U(:,:,r),V(:,:,r),W(:,:,r)},[1 2 3]);
    end
    A0 = A0/frob(A0);
    E = randn(I);
    E = E/frob(E);
    A = A0+sigma*E;
    for s_i = 1 : sample_initial
        % Building the initial iterate(s)
        S0 = randn([Rank R]);
        U0 = zeros(I(1),Rank(1),R);
        V0 = zeros(I(2),Rank(2),R);
        W0 = zeros(I(3),Rank(3),R);
        for r = 1:R
            U0(:,:,r) = qf(randn(I(1),Rank(1)));
            V0(:,:,r) = qf(randn(I(2),Rank(2)));
            W0(:,:,r) = qf(randn(I(3),Rank(3)));
        end
        s = (s_A-1)*sample_initial+s_i; % counter
        % Run btd_grad
        start = tic;
        [S_grad,U_grad,V_grad,W_grad,info_grad] = btd_grad(A,U0,V0,W0,tol,itermax_grad);
        time(s,1) = toc(start);
        normgrad_grad = info_grad.normgrad;
        obj_grad = info_grad.objective;
        % Run btd_grad_full
        start = tic;
        [S_grad_full,U_grad_full,V_grad_full,W_grad_full,info_grad_full] = btd_grad_full(A,S0,U0,V0,W0,tol,itermax_grad_full);
        time(s,2) = toc(start);
        normgrad_grad_full = info_grad_full.normgrad;
        obj_grad_full = info_grad_full.objective;
        % Storing data
        normgrad(s,1) = normgrad_grad(end);
        normgrad(s,2) = normgrad_grad_full(end);
        objective(s,1) = obj_grad(end);
        objective(s,2) = obj_grad_full(end);
        iter(s,1) = length(normgrad_grad);
        iter(s,2) = length(normgrad_grad_full);
        inner_iter(s,1) = mean(info_grad.iterations);
        inner_iter(s,2) = mean(info_grad_full.iterations);
        time(s,3) = info_grad.VarProjTime;
        % Success counter
        if sigma == 0
            if normgrad(s,1) <= tol && objective(s,1) <= tol_obj
                success(1) = success(1)+1;
            end
            if normgrad(s,2) <= tol && objective(s,2) <= tol_obj
                success(2) = success(2)+1;
            end
        else
            if normgrad(s,1) <= tol
                success(1) = success(1)+1;
            end
            if normgrad(s,2) <= tol
                success(2) = success(2)+1;
            end
        end
    end
end
end