function [gU,gV,gW] = grad_g(A,I,R,Rank,S,U,V,W)
% [gU,gV,gW] = grad_g(A,I,R,Rank,S,U,V,W) evaluates the gradient of the cost function g_A.
% INPUT: 
%  - A is the third-order tensor to be approximated by BTD;
%  - I is the size of A;
%  - R is the number of terms in the BTD;
%  - Rank is the multilinear rank of each term;
%  - S is a fourth-order tensor containing the R core tensors;
%  - U is a third-order tensor containing the factor matrices U(:,:,1),...,U(:,:,R);
%  - V is a third-order tensor containing the factor matrices V(:,:,1),...,V(:,:,R);
%  - W is a third-order tensor containing the factor matrices W(:,:,1),...,W(:,:,R).
% OUTPUT:
%  - [gU,gV,gV] is the gradient of g_A.
% matrix representations of A
A1 = tens2mat(A,1,[3,2]);
A2 = tens2mat(A,2,[1,3]);
A3 = tens2mat(A,3,[2,1]);
% matrix representations of S_1,...,S_R
S1 = zeros(Rank(1),Rank(2)*Rank(3),R);
S2 = zeros(Rank(2),Rank(1)*Rank(3),R);
S3 = zeros(Rank(3),Rank(1)*Rank(2),R);
for r = 1:R
    S1(:,:,r) = tens2mat(S(:,:,:,r),1,[3,2]);
    S2(:,:,r) = tens2mat(S(:,:,:,r),2,[1,3]);
    S3(:,:,r) = tens2mat(S(:,:,:,r),3,[2,1]);
end
% gradients with respect to U, V, and W
gU = zeros(I(1),R*Rank(1));
gV = zeros(I(2),R*Rank(2));
gW = zeros(I(3),R*Rank(3));
for i = 1:R % gradient with respect to U_i, V_i, and W_i
    for j = 1:R % loop for the sum
        gU(:,(i-1)*Rank(1)+1:i*Rank(1)) = gU(:,(i-1)*Rank(1)+1:i*Rank(1)) + U(:,:,j)*S1(:,:,j)*kron(V(:,:,j)'*V(:,:,i),W(:,:,j)'*W(:,:,i))*S1(:,:,i)';
        gV(:,(i-1)*Rank(2)+1:i*Rank(2)) = gV(:,(i-1)*Rank(2)+1:i*Rank(2)) + V(:,:,j)*S2(:,:,j)*kron(W(:,:,j)'*W(:,:,i),U(:,:,j)'*U(:,:,i))*S2(:,:,i)';
        gW(:,(i-1)*Rank(3)+1:i*Rank(3)) = gW(:,(i-1)*Rank(3)+1:i*Rank(3)) + W(:,:,j)*S3(:,:,j)*kron(U(:,:,j)'*U(:,:,i),V(:,:,j)'*V(:,:,i))*S3(:,:,i)';
    end
    gU(:,(i-1)*Rank(1)+1:i*Rank(1)) = gU(:,(i-1)*Rank(1)+1:i*Rank(1)) - A1*kron(V(:,:,i),W(:,:,i))*S1(:,:,i)';
    gV(:,(i-1)*Rank(2)+1:i*Rank(2)) = gV(:,(i-1)*Rank(2)+1:i*Rank(2)) - A2*kron(W(:,:,i),U(:,:,i))*S2(:,:,i)';
    gW(:,(i-1)*Rank(3)+1:i*Rank(3)) = gW(:,(i-1)*Rank(3)+1:i*Rank(3)) - A3*kron(U(:,:,i),V(:,:,i))*S3(:,:,i)';
end
gU = 2*gU;
gV = 2*gV;
gW = 2*gW;
end